package dev.micle.xptools.events.common;

import dev.micle.xptools.XpTools;
import dev.micle.xptools.config.Config;
import dev.micle.xptools.operation.OperationCache;
import dev.micle.xptools.operation.OperationItem;
import dev.micle.xptools.operation.OperationUtils;
import net.minecraft.resources.ResourceLocation;
import net.minecraftforge.event.level.BlockEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.registries.ForgeRegistries;

import java.time.Duration;
import java.time.Instant;
import java.util.List;

public class OnBlockBreakEventHandler {
    private final OperationCache cache = OperationCache.blockBreakCache;

    @SubscribeEvent
    public void OnBlockBreakEvent(BlockEvent.BreakEvent event) {
        Instant start = Instant.now();
        float xpToDrop = event.getExpToDrop();

        // Get Block id
        ResourceLocation block_rl = ForgeRegistries.BLOCKS.getKey(event.getState().getBlock());
        String block_id = "";
        if (block_rl != null) {
            block_id = block_rl.toString();
        }

        // Get operations
        List<OperationItem> operations = OperationUtils.getOperationList(
                block_id,
                event.getState().getTags().toList(),
                Config.Server.blockBreakOperationItems,
                cache
        );

        // Add global operations before all others
        operations.addAll(0, Config.Server.blockBreakGlobalOperationItems);

        // Apply operations to xp drops
        xpToDrop = OperationUtils.calculateNewXpAmount(xpToDrop, operations);

        // Debug logging
        if (Config.Server.debugExtra.get()) {
            XpTools.LOGGER.debug("Completed block break event:");
            XpTools.LOGGER.debug("\tOperations: {}", operations);
            XpTools.LOGGER.debug("\tCache entries: {}", cache.size());
            XpTools.LOGGER.debug("\tTime taken (nano seconds): {}", Duration.between(start, Instant.now()).toNanos());
            XpTools.LOGGER.debug("\tXP: {} -> {}", event.getExpToDrop(), xpToDrop);
        }

        // Apply xp drop
        event.setExpToDrop((int)xpToDrop);
    }
}
